close all
clear all
clc

%% APROKSIMACIJA I INTERPOLACIJA 


% ZADATAK 1. (Obratiti pozornost na nazivnik)
% Odredite koeficijente polinoma p = c1*x^5 + c2*x^3 + c3*x + c4 
% kojim cete aproksimirati funkciju y =-x/sqrt(1-x^2) 
% na intervalu (-4/5, 4/5) tako da problem svedete na sustav lin. alg. jed.
x = [-4/5:0.01:4/5]';
f = [-x./sqrt(1-x.^2)];

V = [x.^5, x.^3, x, ones(length(x),1)];
cc1 = V\f;
p = [cc1(1) 0 cc1(2) 0 cc1(3) cc1(4)];

figure(1)
subplot(211)
plot(x, f, x, polyval(p, x), '--r', 'LineWidth',2)
xlabel('x'), ylabel('f(x)')
legend('-x/sqrt(1-x^2)','c_1x^5 + c_2x^3 + c_3x + c_4')

err = polyval(p,x)-f;

subplot(212)
plot(x, err, 'LineWidth',2)
xlabel('x'), ylabel('p(x) - y(x)')
%--------------------------------------------------------------------------


% ZADATAK 2. 
% Odredite polinom kojim cete interpolirati izmjerene
% podatak (xk, fk) rjesavanjem linearnog sustava V*c=fk, gdje je V
% Vandermondova matrica od xk. Rjesenje prikazite graficki. 
xk = [0:5]';
fk = [0 1 2 6 8 7]';

V = vander(xk); 
c2 = V\fk;

xv = [0:0.01:5]';
pv = zeros(length(xv), 1);
for i = 1:length(c2)
    pv = pv + c2(i)*xv.^(length(c2)-i); % ovo moze i s polyval()
end

figure(2)
plot(xk, fk, 'o', xv, pv, 'r')
xlabel('x')
ylabel('f(x)')
legend('Izmjereni podaci', 'Interpolacijski polinom', 'Location', 'NorthWest')
%--------------------------------------------------------------------------


% ZADATAK 3. 
% Odredite polinome 1., 3., 5. i 6. stupnja kojima cete aproksimirati 
% skup izmjerenih podatak (xk, fk) primjenom MATLAB-ove ugradjene funkcije 
% polyfit(). Rjesenje prikazite graficki i za to koristite funkciju 
% polyval().
xk = [-3 -2 -1 0 1 2 3]';
fk = [-0.2774 0.8958 -1.5651 3.4565 3.0601 4.8568 3.8982]';

p1 = polyfit(xk, fk, 1)
p3 = polyfit(xk, fk, 3)
p5 = polyfit(xk, fk, 5)
p6 = polyfit(xk, fk, 6)

xv = -3:0.01:3;

figure(3)
subplot(221), plot(xk, fk, 'o', xv, polyval(p1,xv), 'r')
xlabel('x'), ylabel('f(x)'), title('Polinom 1. stupnja')

subplot(222), plot(xk, fk, 'o', xv, polyval(p3,xv), 'r')
xlabel('x'), ylabel('f(x)'), title('Polinom 3. stupnja')

subplot(223), plot(xk, fk, 'o', xv, polyval(p5,xv), 'r')
xlabel('x'), ylabel('f(x)'), title('Polinom 5. stupnja')

subplot(224), plot(xk, fk, 'o', xv, polyval(p6,xv), 'r')
xlabel('x'), ylabel('f(x)'), title('Polinom 6. stupnja')
%--------------------------------------------------------------------------


% ZADATAK 4. (Pokazati da bazne funkcije ne moraju biti samo polinomi)
% Odredite koeficijente c1, c2 i c3 rjesavanjem linearnog 
% sustava A*c=fk  tako da funkcija f = c1*exp(x) + c2*x*exp(-x) + c3 
% aproksimira podatke (xk, yk). Rjesenje prikazite graficki.
xk = [1.2; 2.8; 4.3; 5.4; 6.8; 7.9];
fk = [7.5; 16.1; 38.9; 67.0; 146.6; 266.2];

A = [exp(xk), xk.*exp(-xk), ones(length(xk),1)];
c4 = A\fk;

xv = 1.2:0.01:7.9;
f = c4(1)*exp(xv) + c4(2)*xv.*exp(-xv) + c4(3);

figure(4)
plot(xk, fk, 'o', xv, f, 'r')
xlabel('x')
ylabel('f(x)')
legend('Izmjereni podaci', 'Aproksimacijska funkcija', 'Location', 'NorthWest')
%--------------------------------------------------------------------------


% ZADATAK 5. (Problem se ne moze zapisati kao linearni sustav)
% Primjenom MATLAB-ove funkcije lsqcurvefit() odredite 
% koeficijente a i b funkcije f = a*exp(b*x) kojom cete aproksimirati
% podatke (xk, fk).

xk = [1.2; 2.8; 4.3; 5.4; 6.8; 7.9];
fk = [7.5; 16.1; 38.9; 67.0; 146.6; 266.2];

% c = [a b]
f = @(c, x) c(1)*exp(c(2)*x); 

c0 = [1, 1];
options = optimset('Display','iter') % obratiti pozornost i na druge opcije
[c6, resnorm] = lsqcurvefit(f, c0, xk, fk, [], [], options);
a = c6(1)
b = c6(2)

xv = xk(1):1e-2:xk(end);
ya = f(c6, xv);

figure(5)
plot(xk, fk, 'o', xv, ya, 'r')
xlabel('x')
ylabel('f(x)')
legend('Izmjereni podaci', 'Aproksimacijska funkcija', 'Location', 'NorthWest')
%--------------------------------------------------------------------------


% ZADATAK 6. (Pade aproksimacija)
% Primjenom MATLAB-ove funkcije lsqcurvefit() odredite Pade-ovu
% aproksimaciju funkcije y=cos(x) na intervalu [-pi/2 pi/2].
x = -pi/2:0.1:pi/2;
y = cos(x);
x0 = [1 0 0 0 1 0 0 0];
%options = optimset('Display','iter')
options = optimoptions('lsqcurvefit','Algorithm','levenberg-marquardt');
[c7, resnorm] = lsqcurvefit(@padeapprox, x0, x, y, [], [], options);

a1 = c7(1)
a2 = c7(2)
a3 = c7(3)
a4 = c7(4)

b1 = c7(5)
b2 = c7(6)
b3 = c7(7)
b4 = c7(8)

fp = padeapprox(c7, x);

figure(6)
plot(x, y, x, fp, '--r', 'LineWidth', 2)
xlabel('x'), ylabel('f(x)'), legend('cos(x)', 'Padeova aproksimacija')
%--------------------------------------------------------------------------


% ZADATAK 7. Primjenom MATLAB-ovih funkcija spline() i interp1() podatke (xk, fk)
% interpolirajte kubicnim splajnom i po dijelovima Hermite-ovim polinomom. 
% Ispisite koeficijente kubicnog splajna. 
xk = 0:5:55;
fk = [0.1 1.884 2.732 2.022 1.65 1.5838 -0.073 -0.002 -0.1122 0.106 1.5265 -0.6321];

p = polyfit(xk, fk, length(xk)-1);

xi = xk(1):1e-2:xk(end);
%yc = interp1(xk, fk, xi, 'spline');
yc = spline(xk, fk, xi);
yh = interp1(xk, fk, xi, 'pchip');
px = polyval(p, xi);

figure(7)
plot(xk, fk, '*', xi, px, 'g', xi, yc, 'r' ,xi, yh, 'k')
xlabel('x'), ylabel('f(x)')
legend('Cvorovi', 'Polinom 11. stupnja', 'Kubicna splajn interpolacija','Po dijelovima Hermite-ova interpolacija')

% xxpp = interp1(xk, fk, 'spline', 'pp');
xxpp = spline(xk, fk)
c8 = xxpp.coefs
%--------------------------------------------------------------------------